#include "popbox.h"

#include <iostream>
#include <qstringlist.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include "popbox.moc"

const int sockTimeout = 10000;

Popbox::Popbox(QString f_popserver, QString f_user, QString f_password)
{
	popserver = f_popserver;
	user = f_user;
	password = f_password;

	timerTimeout = new QTimer(this);
	connect(timerTimeout, SIGNAL(timeout()), this, SLOT(socketTimeout()));
}

/********************************************************************/

Popbox::~Popbox()
{
	delete(popSocket);
	delete(socketBuf);
	delete(serverLookup);
	delete(timerTimeout);
}

/********************************************************************/

void Popbox::changeBox(QString f_popserver, QString f_user, QString f_password)
{
	popserver = f_popserver;
	user = f_user;
	password = f_password;
}

/********************************************************************/

bool Popbox::isOk(QString resp)
{
	if(resp.find("+OK") == -1)
		return false;

	return true;
}

/********************************************************************/

void Popbox::requestPop()
{
	QString buf;

	// Create the socket
	popSocket = new QSocket(this);
	// Signal sent each time it's possible to read something
	connect(popSocket, SIGNAL(readyRead()), this, SLOT(socketRead()));
	connect(popSocket, SIGNAL(connected()), this, SLOT(socketConnected()));

	// Connection to server with hostname resolution
	serverLookup = new QDns(popserver, QDns::A);
	connect(serverLookup, SIGNAL(resultsReady()), this, SLOT(connectServer()));

	state = S_INIT;
}

/********************************************************************/

void Popbox::connectServer()
{
	// Get the IP address of the server
	QValueList<QHostAddress> listServers = serverLookup->addresses();
	if(listServers.isEmpty())
	{
		if(!serverLookup->isWorking())
			emit popResponse(1);
		return;
	}

	// Connect the socket to the server and launch timeout
	timerTimeout->start(sockTimeout, true);
	popSocket->connectToHost(listServers.first().toString(), 110);

	// Buffer to send instructions to the server
	socketBuf = new QTextStream(popSocket);
}

/********************************************************************/

void Popbox::socketTimeout()
{
	// Send an error message
	emit popResponse(5);

	// Close the socket
	popSocket->close();
}

/********************************************************************/

void Popbox::socketConnected()
{
	timerTimeout->stop();
}

/********************************************************************/

void Popbox::socketRead()
{
	int nbMails;

	if (!popSocket->canReadLine())
		return;

	QString responseLine;

	if(state == S_INIT)
	{
		responseLine = popSocket->readLine();
		if(!isOk(responseLine))
		{
			emit popResponse(2);
			return;
		}

		// Connection is Ok (previous test), we send the USER command
		*socketBuf << "USER " << user << "\n";

		state = S_USER;
	}
	else if(state == S_USER)
	{
		responseLine = popSocket->readLine();

		if(!isOk(responseLine))
		{
			emit popResponse(3);
			return;
		}

		// We send the PASS command
		*socketBuf << "PASS " << password << "\n";

		state = S_PASS;
	}
	else if(state == S_PASS)
	{
		responseLine = popSocket->readLine();

		if(!isOk(responseLine))
		{
			emit popResponse(3);
			return;
		}

		// Authentication Ok, we ask for STAT
		*socketBuf << "STAT\n";

		state = S_STAT;
	}
	else if(state == S_STAT)
	{
		responseLine = popSocket->readLine();

		if(!isOk(responseLine))
		{
			emit popResponse(4);
			return;
		}

		// Finally get the f**king number
		nbMails = responseLine.section(0x20, 1, 1).toInt();

		*socketBuf << "QUIT\n";

		state = S_QUIT;

		// Everything went OK, we sent the response
		emit popResponse(0, nbMails);
	}
}
